/* Our objects, the RotoQuestionCone (RQC) and the frame.
   The frame is the eye candy around the RQC. Due to the model used,
   unfortuantely a 'negative space' component of this object overlaps
   the RQC and the RQC cannot receive mouse events. Thus, we make the
   frame itself handle the user's mouse clicks. */

theQC = theStage.getChild( "RotoQuestionCone" );
theFrames = theStage.getChild( "Frames" );

/* Delete the physical model associated with the RQC. */

theQC.deletePhysicalModel();
theQC.qcRotation = 0.0;
theQC.qcDestRotation = 0.0;
theQC.qcVelocity = 0.0;

/* The question being displayed by the RQC.*/

theActiveQuestion = 0;


qcEventHandler = MouseEventHandler();
qcEventHandler.setFilterOnClick( theFrames );
qcEventHandler.onEvent = function( mouseEvent, objectID )
{

/* If the user clicks on the frame before the guide has finished
   giving him/her directions, they obviously know what to do and are
   not interested in hearing him. */

   theWelcomeClipPlayed = true;
   theWelcomeClip.stop();

/* Incremented the target rotation by PI / 6. */

   theQC.qcDestRotation += 1.047198;

/* Increment the active question by 1 (mod 6). */

   theActiveQuestion = (theActiveQuestion + 1) % 6;
}



/* Unfortuantely, Adobe creates an arbitrary 'pivot point' somewhere
   randomly in space around which it rotates an object. This point
   cannot be modified. As a result, the rotation must be composed of
   three transforms:
     - one to move the target object to the origin of the rotation
       axis/plane
     - one to rotate the target along the Y axis in the new coordinate
       space
     - one to re-transform the differently-oriented target object
       back into the original coordinate space */

function turnY( theObj, theAngle )
{
   theObj.pivotRotation = Rotation( 'Y', theAngle );
   theObj.pivotTranslation = Vector(0.0, 0.0, 0.0);
   theObj.pivotTransform = Transform( theObj.pivotTranslation,
      theObj.pivotRotation );
		
   theObj.moveFromPivot = Transform( theObj.worldSpacePosition,
      Rotation( 'Y', 0.0 ) );

   theObj.moveToPivot = theObj.moveFromPivot.inverse;
   theObj.compositeTransform = Transform( theObj.moveFromPivot,
      theObj.pivotTransform, theObj.moveToPivot);

   theObj.worldSpaceTransform = Transform( theObj.compositeTransform,
      theObj.worldSpaceTransform );
}




/* The RQC has a 'spring loaded' second-order LTI physical behaviour.
   The term 'LTI' is used loosely as the system is non-linear across
   the 'target angle has been reached' boundary. */

theQC.timestep = function( t, delta_t )
{

/* If we're zero velocity and target position, drop the cycle... */

   if( theQC.qcVelocity == 0.0 &&
      theQC.qcDestRotation == theQC.qcRotation )
   {
      return;
   }

/* If we're close enough to zero velocity and target position, snap
   the RQC to the correct position and zero the velocity. */

   if( theQC.qcVelocity < 0.02 &&
      Math.abs( theQC.qcDestRotation - theQC.qcRotation ) < 0.02 )
   {
      theQC.qcVelocity = 0.0;
      turnY( theQC, theQC.qcDestRotation - theQC.qcRotation );
      theQC.qcRotation = theQC.qcDestRotation;
      return;
   }

/* Otherwise, rotate the cone with the given velocity. */

   turnY( theQC, delta_t * theQC.qcVelocity );
   theQC.qcRotation += delta_t * theQC.qcVelocity;
   if( theQC.qcRotation > theQC.qcDestRotation )
   {

/* Use one set of acceleration/decay parameters for one when the
   the RQC is past target. These parameters favour 'slingshotting'
   the RQC back toward nominal. */

      theQC.qcVelocity += (theQC.qcDestRotation - theQC.qcRotation) *
         delta_t * 16.;

      theQC.qcVelocity -= theQC.qcVelocity * delta_t * 3.;
   }
   else
   {

/* Use a less pronounced 'spring' and less powerful damper
   otherwise. */

      theQC.qcVelocity += (theQC.qcDestRotation - theQC.qcRotation) *
         delta_t * 4.;

      theQC.qcVelocity -= theQC.qcVelocity * delta_t / 2.;
   }   
}

addAnimator( theQC );