/* Initialize a reference to the guide MTS object and delete the
   physical model so that the orientation can be controlled
   by the script. */

theGuide = theStage.getChild( "amale" );
theGuide.deletePhysicalModel();

/* We want the guide to welcome the user once and only once. */

var theWelcomeClipPlayed = false;

theWelcomeClip = Sound( "../audio/hello.mp3" );
theWelcomeClip.repeats = 1;
theWelcomeClip.position = theGuide.position;
theWelcomeClip.near = 8.0;
theWelcomeClip.far = 20.0;
theWelcomeClip.volume = 1.;

/* The clips for the questions (the simulation is presently hard-coded
   to accept only six question.)  The seventh clip is the .mp3 file
   that is played if the user goes 'click happy' on the guide and the
   guide gets angry. */

theQuestionClipNames = new Array(
   "small.mp3",
   "chickenegg.mp3",
   "santa.mp3",
   "shotjfk.mp3",
   "chuck.mp3",
   "atomicweight.mp3",
   "enjoying.mp3" );

/* Convert the names of the sounds into actual Sound objects. */

theQuestionClips = new Array();
for( i = 0; i < theQuestionClipNames.length; i++ )
{
   theQuestionClips[i] =
      Sound( "../audio/" + theQuestionClipNames[i] );

   theQuestionClips[i].repeats = 1;
   theQuestionClips[i].position = theGuide.position;
   theQuestionClips[i].near = 8.0;
   theQuestionClips[i].far = 20.0;
   theQuestionClips[i].volume = 1.;
}

/* The guide animator should give the guide 'sprite' behaviour.
   The guide will pivot on the (world) Y axis to face the user at all
   times. */

theGuide.timestep = function( t, delta_t )
{
   camera = application.getView(0).getCamera(0);
   lookRay = camera.position.subtract( theGuide.worldSpacePosition );
   theRotation = Rotation( 'Y',
      Math.atan2( lookRay.x, lookRay.z ) );

   theGuide.worldSpaceOrientation = theRotation;
}

addAnimator( theGuide );

/* What happens when we click on the guide? ... */

guideEventHandler = MouseEventHandler();
guideEventHandler.setFilterOnClick( theGuide );
guideEventHandler.onEvent = function( mouseEvent, objectID )
{

/* If we're welcoming the user, stop the welcome. */

   if( theWelcomeClip.active )
      theWelcomeClip.stop();

/* If we're playing any of the question clips, we stop them.
   The 'if( ... active )' clauses are included as a result of the
   Viewpoint player stopping ALL audio clips as the result of
   requesting a non-playing audio clip to stop. */

   for( i = 0; i < 6; i++ )
      if( theQuestionClips[i].active )
         theQuestionClips[i].stop();

/* If the user has overclicked (i.e. gone 'click happy'), we do not
   accept input until the penalty flag has been cleared by the
   counter. */

   if( overclickCounter.penalty )
      return;

/* Each click increments the click counter by one. If the user clicks
   3 times in a 0.5 s period, or 4 times in a 0.5 - 1.5 s period,
   etc., we assume they've gone 'click happy'. */

   overclickCounter.count += 1.0;
   if( overclickCounter.count > 2.6 )
   {

/* Reset the counter. Set the penalty flag. Play the audio message to
   the user. */

      overclickCounter.count = 0.0;
      overclickCounter.penalty = true;
      theQuestionClips[6].play();
      return;
   }

/* If the user clicks on the guide before a welcome has played, we
   play the welcome instead of the question displayed on the RQC.
   This may present if the user clicks on the guide from a distance,
   or if the user approaches the guide, but does not get close enough
   to trigger the proximity sensor implemented below. */

   if( !theWelcomeClipPlayed )
   {
      theWelcomeClip.play();
      runAnimation( 1 );
      theWelcomeClipPlayed = true;
      return;
   }

/* Otherwise, we play the clip associated with the RQC. */

   theQuestionClips[theActiveQuestion].play();
   runAnimation( theActiveQuestion == 5 ? 3 : 2 );
}




/* Create the proximity sensor object. This is simply an animator
   that detects the user's approach to the guide (physical proximity
   and amount of time within an acceptable triggering proximity.) */

var guideProxSensor = new Object();

guideProxSensor.initialTime = -1.;
guideProxSensor.timestep = function( t, delta_t )
{

/* The prox sensor's purpose is to play the welcome clip. If the user
   has invoked this clip from a distance, we can terminate the
   animator. */

   if( theWelcomeClipPlayed )
   {
      removeAnimator( guideProxSensor );
      return;
   }

/* We consider 'within ten feet' to be a reasonable proximity to the
   guide. */

   isProx = camera.position.subtract( theGuide.worldSpacePosition ).
      length < 10 ? true : false;
   if( isProx )
   {

/* If this is the first time the user's camera has fallen within the
   correct proximity, we start a timer. */

      if( guideProxSensor.initialTime == -1.0 )
      {
         guideProxSensor.initialTime = t;
         return;
      }

/* If the user has stayed within the correct proximity for one second
   of time, we trigger the welcome clip. */

      else if( t - guideProxSensor.initialTime > 1.0 )
      {
         removeAnimator( guideProxSensor );
         theWelcomeClip.play();
         runAnimation( 1 );
         theWelcomeClipPlayed = true;
      }
   }

/* If the user walks out of the area of proximity again before the
   clip is played (i.e. user backs away or is just walking by), the
   timer is reset. */

   else
      guideProxSensor.initialTime = -1.0;
}

addAnimator( guideProxSensor );



/* The overclick counter facilitates detection of the user going
   'click happy' on the guide. Clicking on the guide rapidly in
   succession makes him upset, and he'll let the user know this.
   The 'timestep()' function of the object also serves to prevent
   the user from doing anything else while the guide is ranting. */

var overclickCounter = new Object();

overclickCounter.penaltyStartTime = -1.;
overclickCounter.penalty = false;
overclickCounter.count = 0.0;
overclickCounter.timestep = function( t, delta_t )
{

/* If we're in 'penalty' mode (i.e. the user has already tripped the
   counter and the timestep() function is now acting in 'prevent
   user from doing anything else' mode. */

   if( overclickCounter.penalty )
   {

/* If the timer hasn't started, we start it. */

      if( overclickCounter.penaltyStartTime == -1. )
         overclickCounter.penaltyStartTime = t;
      else

/* If more than two seconds has elapsed on the penalty mode (roughly
   the length of the annoyed guide audio clip), we clear the penalty
   flag and resume normal operation. */

         if( t - overclickCounter.penaltyStartTime > 2.0 )
         {
            overclickCounter.penalty = false;
            overclickCounter.penaltyStartTime = -1.;
         }

      return;
   }

/* In normal mode, we reduce the overclick counter by one click per
   two seconds. Basically, if the user clicks less frequently than
   once every two seconds, the counter will effectively not be
   incremented and the guide will never get annoyed. */

   overclickCounter.count -= delta_t / 2.;
   if( overclickCounter.count < 0.0 )
      overclickCounter.count = 0.0;
}

addAnimator( overclickCounter );



animation = new Array("Stand","Action_1","Action_4","Action_2");
insName   = "amale";  

topInstance = false;
animRef     = new Array(animation.length);
scene       = this.scene;
animSpeed   = 1.1;
numChildren	= new Array();
childCount	= new Array();



function findNamedTimeElem( elemName )
{
    numTimeElems = scene.getTimeElemCount();
    for (i = 0; i < numTimeElems; i++)
    {
        timeElem = scene.getTimeElemFromIndex(i);
        if (timeElem.getName() == elemName) { return timeElem };
        var childCount = timeElem.getChildCount();
        for (j = 0; j < childCount; j++)
        {
            child = timeElem.getChild(j);
            if (child.getName() == elemName) { return child };
            var childCount2 = child.getChildCount();
            for (k = 0; k < childCount2; k++)
            {
                child2 = child.getChild(k);
                if (child2.getName() == elemName) { return child2 };
            }
        }
    }
}

function assignAnimations()
{
    for (m = 0; m < animation.length ; m++)
    {
        animRef[m] = findNamedTimeElem( animation[m] );
    }
}

function runAnimation( ref )
{
    for (n=0; n < animation.length; n++)
    {
        animRef[n].stop();
    }
	animRef[ref].setSpeed(animSpeed);
	animRef[0].trigger();
    animRef[ref].trigger();
}


loadCheck = new Object();
loadCheck.timestep = function( now )
{
    if (!topInstance)
    {
        topInstance = scene.getInstance( insName );
    }
    else if ( animRef[animation.length - 1] == null )
    {
		assignAnimations();
    }
	else
    {
		removeAnimator(loadCheck);
    }
}

addAnimator(loadCheck);